﻿using System;
using System.IO;

namespace TabSaver.ToolbarButtons
{
    /// <summary>
    /// This is a sample for toolbar button acting as COMObject.
    /// It'll appear in the tools menu of the ie and has specific behaviour 
    /// which can be influenced by setting different properties
    /// especially the AssociatedCLSIDExtension which is the guid of the target COMObject to launch
    /// </summary>
    public class LoadTabsObjectButton : SpicIE.Controls.ToolbarButton
    {
        /// <summary>
        /// The PluginGuid _must_ be a unique GUID to identify and register the COM object correctly
        /// </summary>
        public override string PluginGuid
        {
            get { return "BE9B2F7D-BBB3-498f-9F3C-08E21BC8884A"; }
        }

        /// <summary>
        /// The PluginProgID _must_ be provided to identify and register the COM object correctly
        /// </summary>
        public override string PluginProgID
        {
            get { return "TabSaver.LoadTabsObjectButton"; }
        }

        public LoadTabsObjectButton()
        {
            this.ButtonName = "Quick Load saved Tabs";
            this.ButtonText = "Quick Load saved Tabs";
            this.MenuText = "Quick Load saved Tabs";
            this.MenuStatusBar = "Quick Load all Tabs that were saved previously";
            this.ButtonAppearance = ButtonAppearanceEnum.COMObject;

            // GetActivePath() represents the current directory where the plugin is located (and the ressources should be, too)
            // with HotIconPath you set the colored, active icon for the toolbar button
            // with Iconpath you set the grayscaled, inactive icon  for the toolbar button
            this.HotIconPath = Path.Combine(SpicIE.Host.GetActivePath() + Path.DirectorySeparatorChar + "Resources", "Load.ico");
            this.IconPath = Path.Combine(SpicIE.Host.GetActivePath() + Path.DirectorySeparatorChar + "Resources", "Load.ico");

            this.ButtomCustomMenu = ButtonCustomEnum.Tools;

            // here you specify the CLSID of the COM object you want to invoke
            // the COM object which shall be invoke, needs to implement the IOleCommandTarget interface
            // in this case the SimpleCOMObject is referenced here, to demonstrate this functionality
            this.AssociatedCLSIDExtension = new Guid("09AFB878-CC96-42bc-9ED7-6CD62C3C10BF");
        }
    }
}